@@ -1,5 +1,329 @@
-corrMM.LRT <-
-function(null.formula=NULL,formula,
+`Bartlett.robust` <- function(LRTobject,robust=T,verbose=F) {
+      bootLRTS <- with(LRTobject,2*(bootreps[,1]-bootreps[,2]))## full -null but can be p_v or p_bv
+      # plot(qchisq(ppoints(zut),1),sort(zut)) ## QQplot, MASS p. 108
+      filter <- bootLRTS[bootLRTS>-1e-08]
+      resu <- list()
+      if (robust) {
+        ## finds the df of the distribution by robust regression (MM: MASS p. 161) of QQplot, 
+        robustMean <- rlm(sort(filter)~qchisq(ppoints(filter),1)-1,method="MM",maxit=200)$coefficients[[1]] 
+        ## [[1]] to remove name which otherwise finishes as a rowname in a subsequent dataframe       
+        # plot(qchisq(ppoints(filter),1),sort(filter)) ## QQplot, MASS p. 108
+        # points(qchisq(ppoints(filter),1),robustMean * qchisq(ppoints(filter),1),pch=".")   
+        if (class(robustMean)=="try-error") {
+          resu$robustMean <- NA
+          mess <- pastefrom("problem in computation of robustMean.")
+          warning(mess)
+        } else resu$robustMean <- robustMean
+      }
+      resu$meanPosbootLRT <- mean(filter)
+      resu$nPosbootLRT <- length(filter) 
+      if (verbose) print(unlist(resu))
+      return(resu)
+}
+
+## fixedLRT is a safe interface for performing tests as described in Ecography paper.
+## it does not allow the profiling procedure in corrMM.LRT
+fixedLRT <- function(null.formula,formula,data,HLmethod,REMLformula=NULL,boot.repl=0,
+                        control=list(),control.boot=list(),...) {  ## since corrMM.LRT is not doc'ed, REMLformula=NULL,boot.repl=0 cannot go into '...' 
+  if (missing(null.formula)) stop("'null.formula' argument is missing, with no default.")
+  if (missing(formula)) stop("'formula' argument is missing, with no default.")
+  if (missing(data)) stop("'data' argument is missing, with no default.")
+  if (missing(HLmethod)) stop("'HLmethod' argument is missing, with no default.")
+  if (! is.null(control$profiles)) {
+    stop("'fixedLRT' does not allow 'control$profiles'.")
+  }
+  ## see 'lm' code for template
+  mc <- match.call(expand.dots = TRUE)
+  ## other possible settings, through iterative fits
+  #  mc$init.corrHLfit$lambda <- NULL
+  #  mc$init.corrHLfit$phi <- NULL
+  ## we have a potential backward compatiblity problem, since the simulation scripts 
+  ## for the Ecography paper assume that the package automatically interpret the model as spatial, even if findSpatial returns NULL
+  ## and we no longer want such a behaviour
+  ## but fixedLRT is not used in these scripts, so it can make a different assumption
+  spatial <- findSpatial(formula)
+  if ( ! is.null(spatial)) {
+    mc$method <- "corrHLfit" ## useful for spaMMLRT call but not for corrMM.LRT where it is the default
+    ## both will use p_v for the optim steps, we need to distinguish whether some REML correction is used in iterative algo :
+    if ( HLmethod %in% c("ML","PQL/L","SEM") || substr(HLmethod,0,2) == "ML") {
+      mc[[1L]] <- as.name("spaMMLRT") ## does not (yet) handles well other HLmethod's  when eg init.corrHLfit contains lambda
+      ## there's no profile etc in spaMMLRT... 
+    } else { ## EQL, REPQL or REML variants: profiles then not allowed within corrMM.LRT!
+      mc[[1L]] <- as.name("corrMM.LRT") ## corrMM.LRT methods and its options below are best frozen to their v1.0 state
+      mc$control<-list(profiles=0,prefits=FALSE) ## default values in call by fixedLRT. corrMM.LRT further has default restarts=TRUE and maxit=1
+      mc$control[names(control)]<-control ## overrides with user values
+      mc$control.boot <- control.boot ## default values in call by fixedLRT are those of corrMM.LRT ie prefits=FALSE,profiles=0. We can directly copy user values. 
+    }
+  } else {
+    mc[[1L]] <- as.name("spaMMLRT") 
+    ## No profiles, maxit, restarts, prefits
+    if (is.null(mc$corrMatrix)) { ## neither explicit spatial nor corrMatrix -> HLfit
+        mc$method <- "HLfit"
+      } else {
+      ## corrMatrix -> we need to use HLCor
+        mc$method <- "HLCor"
+      }                 
+  }  
+  eval(mc, parent.frame())
+}
+
+
+spaMMLRT <- function(null.formula=NULL,formula,
+                     null.disp=list(),REMLformula=NULL,
+                     method="corrHLfit",boot.repl=0,
+                     ## currently trace always false; this is not an argument t be forwarded as is to corrHLfit! 
+                     trace=FALSE, ## T means lead to calls of corrHLfit(... trace=list(<file name>,<over/append>))
+                     verbose=c(trace=FALSE,warn=NA,summary=FALSE),  
+                     ...) {
+  if (is.na(verbose["trace"])) verbose["trace"] <- FALSE
+  if (is.na(verbose["warn"])) verbose["warn"] <- FALSE ## will be unconditionally ignored by the final fit in corrHLfit  
+  if (is.na(verbose["summary"])) verbose["summary"] <- FALSE ## this is for HLCor
+  dotlist <-list(...)
+  ## birth pangs :
+  if ("predictor" %in% names(dotlist)) {
+    stop("'spaMMLRT' called with 'predictor' argument which should be 'formula'" )
+  }
+  if ("null.predictor" %in% names(dotlist)) {
+    stop("'spaMMLRT' called with 'null.predictor' argument which should be 'null.formula'" )
+  }
+  ## here we makes sure that *predictor variables* are available for all data to be used under both models
+  data <- dotlist$data
+  if ( inherits(data,"list")) {
+    data <- lapply(data,function(dt) {
+      null.validdata <- validData(formula=null.formula[-2],resid.formula=dotlist$resid.formula,data=dt) ## will remove rows with NA's in required variables
+      full.validdata <- validData(formula=formula[-2],resid.formula=dotlist$resid.formula,data=dt) ## will remove rows with NA's in required variables
+      dt[intersect(rownames(null.validdata),rownames(full.validdata)),,drop=FALSE]     
+    })
+  } else {
+    null.validdata <- validData(formula=null.formula[-2],resid.formula=dotlist$resid.formula,data=data) ## will remove rows with NA's in required variables
+    full.validdata <- validData(formula=formula[-2],resid.formula=dotlist$resid.formula,data=data) ## will remove rows with NA's in required variables
+    data <- data[intersect(rownames(null.validdata),rownames(full.validdata)),,drop=FALSE]     
+  }  
+  dotlist$data <- data
+  predictor <- formula   
+  if (! "predictor" %in% class(formula)) predictor <- Predictor(formula)
+  null.predictor <- null.formula   
+  if (! "predictor" %in% class(null.formula)) null.predictor <- Predictor(null.formula)
+  form <- predictor
+  if (!is.null(dotlist$LamFix)) {
+    dotlist$ranFix$lambda <- dotlist$LamFix
+    dotlist$LamFix <- NULL
+  }
+  if (!is.null(dotlist$PhiFix)) {
+    dotlist$ranFix$phi <- dotlist$PhiFix
+    dotlist$PhiFix <- NULL
+  }  
+  dotlist$formula <- predictor 
+  dotlist$verbose <- verbose 
+  if (method=="corrHLfit") {
+    if (is.null(dotlist$objective)) { ## implements default objective function
+      if ( ! is.null(null.predictor)) {dotlist$objective <- "p_v"} else {dotlist$objective <- "p_bv"}
+    }
+  } else dotlist$objective <- NULL
+  ## do not change dotlist afterwards !
+  fullm.list <- dotlist
+  nullm.list <- dotlist
+  fullm.list$formula <- predictor
+  #### "limited use, for initializing bootstrap replicates:"
+  which.iterative.fit <-character(0)
+  which.optim.fit <-character(0)
+  if ( ! is.null(dotlist$init.corrHLfit$lambda) ) {
+    which.optim.fit <- c(which.optim.fit,"lambda")
+  } else if ( is.null (fullm.list$ranFix$lambda)) which.iterative.fit <- c(which.iterative.fit,"lambda")
+  if (is.null(fullm.list$family) ## (=gaussian)
+      || fullm.list$family$family %in% c("gaussian","Gamma")) {
+    if ( ! is.null(dotlist$init.corrHLfit$phi) ) { ## if user estimates it by ML...
+      which.optim.fit <- c(which.optim.fit,"phi")
+    } else which.iterative.fit <- c(which.iterative.fit,"phi")
+  }
+  if (method=="corrHLfit") {
+    if ( ! "rho" %in% names(dotlist$ranFix)) which.optim.fit <- c(which.optim.fit,"rho")
+    if ( ! "nu" %in% names(dotlist$ranFix)) which.optim.fit <- c(which.optim.fit,"nu")
+    if ( ! "ARphi" %in% names(dotlist$ranFix)) which.optim.fit <- c(which.optim.fit,"ARphi") # mpf
+    if ( ! "Nugget" %in% names(dotlist$ranFix)) which.optim.fit <- c(which.optim.fit,"Nugget") ## by default not operative given later code and init.optim$Nugget is NULL
+  }
+  if ( ! is.null(null.predictor)) { ## ie if test effet fixe
+    testFix <- T
+    if (dotlist$HLmethod =="SEM") {
+      test.obj <- "logLsmooth"
+    } else test.obj <- "p_v"
+    ## check fullm.list$REMLformula, which will be copied into nullm in all cases of fixed LRTs
+    if (dotlist$HLmethod %in% c("ML","PQL/L","SEM") || substr(dotlist$HLmethod,0,2) == "ML") {
+      fullm.list$REMLformula <- NULL
+      nullm.list$REMLformula <- NULL
+    } else { ## an REML variant
+      if (is.null(REMLformula)) { ## default
+        fullm.list$REMLformula <- NULL ## HLfit will reconstruct proper REML from this
+        nullm.list$REMLformula <- fullm.list$formula ## 
+      } else { ## allows alternative choice, but still the same *both* the full and the null fit
+        fullm.list$REMLformula <- REMLformula
+        nullm.list$REMLformula <- REMLformula
+      }
+      namesinit <- names(fullm.list$init.corrHLfit)
+      namesinit <- setdiff(namesinit,c("rho","nu","Nugget","ARphi"))
+      len <- length(namesinit)
+      if ( len>0) {
+        if (len > 1) namesinit <- paste(c(paste(namesinit[-len],collapse=", "),namesinit[len]),collapse=" and ")
+        message("Argument 'init.corrHLfit' is used in such a way that")
+        message(paste("  ",namesinit," will be estimated by maximization of p_v.",sep=""))
+        message("  'REMLformula' will be inoperative if all dispersion")
+        message("  and correlation parameters are estimated in this way.")
+      }
+    }
+    ############## deja converti if (! "predictor" %in% class(null.formula)) null.predictor <- Predictor(null.predictor)
+    nullm.list$formula <- null.predictor
+  } else if ( length(null.disp)>0 ) { ## test disp/corr param
+    testFix <- F
+    test.obj <- "p_bv"
+    #
+    mess <- pastefrom("changes to objective fn required to renullfit or refull fit computation.")
+    stop(mess)
+    #
+    namescheck <- names(fullm.list$lower)
+    namescheck <- namescheck[ ! namescheck %in% names(null.disp)]  
+    nullm.list$lower <- nullm.list$lower[namescheck]
+    nullm.list$upper <- nullm.list$upper[namescheck]
+    nullm.list$ranFix <- c(nullm.list$ranFix,null.disp) ## adds fixed values to any preexisting one
+  } else testFix <- NA
+  
+#  trace.info <- NULL
+  fullfit <- do.call(method,fullm.list)
+  nullfit <- do.call(method,nullm.list)
+  if (testFix) {df <- length(fullfit$fixef)-length(nullfit$fixef)} else {df <- length(null.disp)}
+  if (df<0) {
+    tmp <- fullfit
+    fullfit <- nullfit
+    nullfit <- tmp
+  }
+  if (inherits(fullfit,"HLfitlist")) {
+    fullL <- attr(fullfit,"APHLs")[[test.obj]]
+  } else fullL <- fullfit$APHLs[[test.obj]]
+  if (inherits(nullfit,"HLfitlist")) {
+    nullL <- attr(nullfit,"APHLs")[[test.obj]]
+  } else nullL <- nullfit$APHLs[[test.obj]]
+  LRTori <- 2*(fullL-nullL)
+  ## BOOTSTRAP
+  if ( ! is.na(testFix)) {
+    if (boot.repl>0) {
+      bootlist <- dotlist ## copies ranFix
+      bootlist <- c(bootlist,list(null.formula=null.predictor,null.disp=null.disp,REMLformula=REMLformula,method=method)) ## unchanged user REMLformula forwarded
+      bootlist$verbose <- c(trace=FALSE,summary=FALSE)
+      bootlist$trace <- FALSE 
+      bootlist$boot.repl <- 0 ## avoids recursive call of bootstrap
+      all.estim.ranvars <- c(which.optim.fit) ## FR->FR to be modified if optimFits are reintroduced, see corresponding code in corrMM.LRT 
+      for (st in all.estim.ranvars) {
+        if (dotlist$HLmethod=="corrHLfit") {
+          if ( st %in% names(nullfit$corrPars)) {
+            bootlist$init.corrHLfit[st] <- nullfit$corrPars[st]
+          } else if ( st %in% names(nullfit)) {
+            bootlist$init.corrHLfit[st] <- nullfit[st] ## handled in dotlist by corrHLfit, cf notes 090113
+          } ## it's also possible that st is nowhere (10/2013: currently for Nugget)
+        } else {
+          if ( st %in% names(nullfit)) {
+            bootlist$init.HLfit[st] <- nullfit[st] ## 
+          } 
+        }
+      }        
+      bootreps<-matrix(,nrow=boot.repl,ncol=2) 
+      colnames(bootreps) <- paste(c("full.","null."),test.obj,sep="")
+      msg <- "bootstrap replicates: "
+      msglength <- nchar(msg)
+      cat(msg)
+      t0 <- proc.time()["user.self"]
+      simbData <- nullfit$data
+      if (tolower(nullfit$family$family)=="binomial") {
+        form <- attr(nullfit$predictor,"oriFormula") ## this must exists...  
+        if (is.null(form)) {
+          mess <- pastefrom("a 'predictor' object must have an 'oriFormula' member.",prefix="(!) From ")
+          stop(mess)
+        }
+      }
+      ## the data contain any original variable not further used; e.g original random effect values in the simulation tests  
+      thisFnName <- as.character(sys.call()[[1]]) ## prevents a bug when we change "this" function name
+      for (ii in 1:boot.repl) {
+        locitError <- 0
+        repeat { ## for each ii!
+          newy <- simulate(nullfit) ## cannot simulate all samples in one block since some may not be analyzable  
+          if (tolower(nullfit$family$family)=="binomial") {
+            ## c'est bouseux: soit j'ai (pos, neg) et le remplacement est possible
+            ##    soit j'ai (pos,ntot -pos) et le 2e remplacment n'est pas poss (et pas necess)
+            ##    aussi (ntot - pos, pos) ...
+            ## would be simple if always ntot-pos, but how to control this ? 
+            ## simbData[[as.character(form[[2]][[2]])]] <- newy
+            ## simbData[[as.character(form[[2]][[3]])]] <- nullfit$weights - newy    
+            exprL <- as.character(form[[2]][[2]]) 
+            exprR <- as.character(form[[2]][[3]]) 
+            if (length(exprL)==1) simbData[[exprL]] <- newy 
+            if (length(exprR)==1) simbData[[exprR]] <- nullfit$weights - newy                    
+            ## if (length(exprR)! =1) exprRdoes not correspond to a column in the data;frmae so there is no column to replace                     
+          } else {simbData[[as.character(nullfit$predictor[[2]])]] <- newy}
+          bootlist$data <- simbData
+          bootrepl <- try(do.call(thisFnName,bootlist)) ###################### CALL ##################
+          if (class(bootrepl)[1] != "try-error") { ## eg separation in binomial models... alternatively, test it here (require full and null X.pv... )
+            if (inherits(bootrepl$fullfit,"HLfitlist")) {
+              fullL <- attr(bootrepl$fullfit,"APHLs")[[test.obj]]
+            } else fullL <- bootrepl$fullfit$APHLs[[test.obj]]
+            if (inherits(bootrepl$nullfit,"HLfitlist")) {
+              fullL <- attr(bootrepl$nullfit,"APHLs")[[test.obj]]
+            } else fullL <- bootrepl$nullfit$APHLs[[test.obj]]
+            bootreps[ii,] <- c(bootrepl$fullfit$APHLs[[test.obj]],bootrepl$nullfit$APHLs[[test.obj]])
+            break ## replicate performed, breaks the repeat
+          } else { ## there was one error
+            locitError <- locitError + 1
+            if (locitError>10) { ## to avoid an infinite loop
+              stop("Analysis of bootstrap samples fails repeatedly. Maybe no statistical information in them ?")
+            } ## otherwise repeat!
+          }
+        } 
+        tused <- proc.time()["user.self"]-t0
+        ttotal <- tused* boot.repl/ii
+        if (interactive()) {
+          for (bidon in 1:msglength) cat("\b")
+          msg <- paste("Estimated time remaining for bootstrap: ",signif(ttotal-tused,2)," s.",sep="")
+          msglength <- nchar(msg)
+          cat(msg)
+        } else {
+          cat(ii);cat(" ")
+          if ((ii %% 40)==0) cat("\n")
+        }
+      }
+      cat("\n")
+    } ## end main bootstrap loop
+  } else { ## nothing operativ yet
+    bootreps<-matrix(,nrow=boot.repl,ncol=length(unlist(fullfit$APHLs))) 
+    colnames(bootreps) <- names(unlist(fullfit$APHLs))
+    ## more needed here ?
+  }
+  ## prepare output
+  if ( ! is.na(testFix)) {
+    if (testFix) {df <- length(fullfit$fixef)-length(nullfit$fixef)} else {df <- length(null.disp)}
+    resu <- list(fullfit=fullfit,nullfit=nullfit)
+    LRTinfo <- list(df=df,LRTori = LRTori)
+    basicLRT <- data.frame(LR2=LRTori,df=df,pvalue=1-pchisq(LRTori,df=df))
+    if (boot.repl>0) {
+      bootdL <- bootreps[,1]-bootreps[,2]
+      meanbootLRT <- 2*mean(bootdL)  
+      LRTinfo$rawPvalue <- (1+sum(bootdL>=LRTori/2))/(boot.repl+1) ## DavisonH, p.141
+      LRTinfo$meanbootLRT <- meanbootLRT
+      LRTinfo$bootreps <- bootreps
+      LRTinfo$LRTcorr <- LRTori*df/meanbootLRT
+    }
+  } else {
+    resu <- list(fullfit=fullfit)
+    LRTinfo <- list()
+    basicLRT <- list()
+  }
+  #  LRTinfo$trace.info <- trace.info 
+  ##  resu$LRTinfo <- LRTinfo ## pas compatible avec hglmjob.R...
+  resu <- c(resu,LRTinfo) ## loses the sublist structure, which wouldnot be compatible with hglmjob.R...  
+  ## this keeps the dataframe structure of basicLRT:
+  resu <- c(resu,list(basicLRT = basicLRT)) ## as in anova and LRT and their summaries  
+  class(resu) <- c("fixedLRT",class(resu)) 
+  return(resu)
+}
+
+
+`corrMM.LRT` <- function(null.formula=NULL,formula,
                        null.predictor=null.formula,predictor=formula, ## simple back compat code...
                        null.disp=list(),REMLformula=NULL,
                        method="corrHLfit",boot.repl=0,
@@ -44,10 +368,6 @@ function(null.formula=NULL,formula,
   if (! "predictor" %in% class(predictor)) predictor <- Predictor(predictor)
   form <- predictor
   dotlist <-list(...)
-#  if (!is.null(dotlist[["always.prof.fixed"]])) { ## option completement obsolete mais indicative du  script publie ECOGRAPHY hence hack:
-#    dotlist$control.HLfit$version <- "1.0"
-#    dotlist[["always.prof.fixed"]] <- NULL ## et un petit menage au passage
-#  }
   if (!is.null(dotlist$LamFix)) {
       dotlist$ranFix$lambda <- dotlist$LamFix
     dotlist$LamFix <- NULL
@@ -228,12 +548,18 @@ function(null.formula=NULL,formula,
         }    
      }  
      if ("nu" %in% which.pars) {
-        if ("nu" %in% which.optim.fit) { ## always by ML whether user provided initial values or not
-          to.arglist$ranFix$nu <- from.fit$corrPars$nu   
-          to.arglist$init.corrHLfit$nu <- NULL
-        }
+       if ("nu" %in% which.optim.fit) { ## always by ML whether user provided initial values or not
+         to.arglist$ranFix$nu <- from.fit$corrPars$nu   
+         to.arglist$init.corrHLfit$nu <- NULL
+       }
      }      
-    return(to.arglist)
+     if ("Nugget" %in% which.pars) {
+       if ("Nugget" %in% which.optim.fit) { ## always by ML whether user provided initial values or not
+         to.arglist$ranFix$Nugget <- from.fit$corrPars$Nugget   
+         to.arglist$init.corrHLfit$Nugget <- NULL
+       }
+     }      
+     return(to.arglist)
   }  
   ####
   #### ALWAYS p_v for testing fixed effects, p_bv for random effects 
@@ -339,11 +665,11 @@ function(null.formula=NULL,formula,
     fullfit <- lamREMLfullfit
   }
   ## ITERATIONS
-  nullnamesX <- colnames(nullfit$X)
-  fullnamesX <- colnames(fullfit$X)
+  nullnamesX <- colnames(nullfit$`X.pv`)
+  fullnamesX <- colnames(fullfit$`X.pv`)
   nullVarsPos <- which( fullnamesX %in% nullnamesX) 
   profileVars <- which( ! fullnamesX %in% nullnamesX) ## les parametres testes
-  profileX <- fullfit$X[,profileVars,drop=F]
+  profileX <- fullfit$`X.pv`[,profileVars,drop=F]
   locit <- 1
   if (! (optimFits && locit <= maxit) ) { ## will not enter the loop => LRTori ici
     LRTori <- 2*(fullfit$APHLs[[test.obj]]-nullfit$APHLs[[test.obj]])
@@ -499,9 +825,9 @@ if (restarts) {
         }
         profilize <- function(offsetbeta) {
           addOffset <- profileX %*% offsetbeta ## semble marcher pour scalaire comme pour vecteur
-          offset <- attr(prof.list$formula,"offset")
-          if ( is.null(offset)) offset <- 0
-          attr(prof.list$formula,"offset") <- offset + addOffset
+          offset <- attr(prof.list$formula,"offsetObj")$vector
+          if ( is.null(offset)) stop("problem with offset in profiling code")
+          attr(prof.list$formula,"offsetObj") <- list(vector=offset + addOffset,nonZeroInfo=TRUE)
           prof.fit <- do.call(method,prof.list) ## 
           return(prof.fit$APHLs[[test.obj]])
         }
@@ -534,9 +860,9 @@ if (restarts) {
           if (interval[2]<0) interval[2] <- beta_se[profileVars]
           profmax <- optimize(profilize,interval=interval,maximum=T)
           addOffset <- profileX %*% profmax$maximum
-          offset <- attr(prof.list$formula,"offset")
-          if ( is.null(offset)) offset <- 0 ## ? FR->FR code repetitif, creer update.offset
-          attr(prof.list$formula,"offset") <- offset + addOffset
+          offset <- attr(prof.list$formula,"offsetObj")$offset
+          if ( is.null(offset)) stop("problem with offset in profiling code") ## ? FR->FR code repetitif, creer update.offset
+          attr(prof.list$formula,"offsetObj") <- list(vector=offset + addOffset,nonZeroInfo=TRUE)
           proffit <- do.call(method,prof.list) ## to recover the ranef parameters etc
           if ( proffit$APHLs[[test.obj]] > fullfit$APHLs[[test.obj]] ) { ## if profile max should improve fullfit
             trace.info <- rbind(trace.info,data.frame(iter=locit,step="proffit (+)",obj=proffit$APHLs[[test.obj]]))
@@ -652,10 +978,9 @@ if (restarts) {
     colnames(bootreps) <- names(unlist(fullfit$APHLs))
     ## more needed here ?
   }
-  ## prepare output
+  ## prepare output (messy for back compatibility)
   if ( ! is.na(testFix)) {
     if (testFix) {df <- length(fullfit$fixef)-length(nullfit$fixef)} else {df <- length(null.disp)}
-    ## pvalue <- 1-pchisq(LRT,df=df)
     resu <- list(fullfit=fullfit,nullfit=nullfit)
     LRTinfo <- list(df=df,LRTprof = LRTprof,LRTori = LRTori)
     if (boot.repl>0) {
@@ -663,13 +988,46 @@ if (restarts) {
       LRTinfo$meanbootLRT <- meanbootLRT
       LRTinfo$bootreps <- bootreps
     }
+    basicLRT <- data.frame(LR2=LRTori,df=df,pvalue=1-pchisq(LRTori,df=df))
   } else {
     resu <- list(fullfit=fullfit)
     LRTinfo <- list()
+    basicLRT <- list()
   }
   LRTinfo$trace.info <- trace.info 
 ##  resu$LRTinfo <- LRTinfo ## pas compatible avec hglmjob.R...
   resu <- c(resu,LRTinfo) ## loses the sublist structure, which wouldnot be compatible with hglmjob.R...  
+  resu <- c(resu,list(basicLRT = basicLRT)) ## as in anova and LRT and their summaries  
   class(resu) <- c("fixedLRT",class(resu)) 
   return(resu)
 }
+
+summary.fixedLRT <- function(object,verbose=TRUE,...) {
+  LRT <- object$basicLRT
+  print(object$basicLRT)
+  #   if (verbose) {
+  #     cat(" ========      'full' model:     ========\n")    
+  #     summary(object$fullfit,...) 
+  #     cat(" ========      'null' model:     ========\n")    
+  #     summary(object$nullfit,...) 
+  #     cat(" ======== Likelihood ratio test: ========\n")    
+  #   }
+  #   outst <- paste(" LR statistic (",object$df," df): ",signif(object$LRTori,3),sep="")    
+  #   cat(outst)
+  bootInfo <- object$bootInfo
+  if (!is.null(bootInfo)) {
+    cat(" ======== Bootstrap: ========\n")    
+    outst <- paste("Raw simulated p-value: ",signif(bootInfo$rawPvalue,3),sep="")    
+    cat(outst)
+    X2 <- bootInfo$LRTcorr
+    outst <- paste("\nBartlett-corrected LR statistic (",bootInfo$df," df): ",signif(X2,3),sep="")    
+    cat(outst)
+  } 
+}
+
+print.fixedLRT <-function(x,...) {
+  summary(x,...)
+  invisible(x)
+}
+
+    
